@@ -2,17 +2,21 @@
 # Author: Jakub Tomasek (jakub@cognicept.systems)
 # --> AgentLifeCycle handles life cycle of Cognicept Agents
 
+import time
 import docker
 import boto3
 import base64
 import json
 import os
+import subprocess
 import dateutil
 from datetime import datetime
 import re
 import glob
 import requests
+import botocore
 from cogniceptshell.common import bcolors
+from subprocess import DEVNULL
 
 class AgentLifeCycle:
 
@@ -78,7 +82,7 @@ class AgentLifeCycle:
                     cgs_agent_status = bcolors.WARNING + "STALE" + bcolors.ENDC
 
         except:
-            cgs_agent_status = bcolors.FAIL + "Error" +  bcolors.ENDC;
+            cgs_agent_status = bcolors.FAIL + "Error" +  bcolors.ENDC
 
         
 
@@ -153,17 +157,59 @@ class AgentLifeCycle:
                 break
         print(output_text)
 
-    def restart(object, args):
-        print("Restarting agents")
-        object.remove_agents(args)
-        result = object.run(args)
+    def restart(object, args):        
+        object.stop(args)
+        result = object.start(args)
         return result
 
-    def remove_agents(object, args):   
+    def start(object, args):        
+        if(args.agents or (hasattr(args, 'list') and len(args.list) > 0)):
+            print("Starting agents")
+            result = object.run_agents(args)
+        elif(args.datadog):
+            print("Starting datadog")
+            result = object.run_datadog(args)
+        else:
+            print("Starting agents") 
+            result_agents = object.run_agents(args)
+            print("Starting datadog")
+            result_datadog = object.run_datadog(args)
+            result = result_agents and result_datadog
+        
+        return result
+
+
+    def stop(object, args):
+        
+        if(args.agents or (hasattr(args, 'list') and len(args.list) > 0)):
+            print("Stopping agents")
+            result = object.remove_agents(args)
+        elif(args.datadog):
+            print("Stopping datadog")
+            result = object.stop_datadog(args)
+        else:
+            print("Stopping agents")
+            result_agents = object.remove_agents(args)
+            print("Stopping datadog")
+            result_datadog = object.stop_datadog(args)
+            result = result_agents and result_datadog
+        return result
+
+    def status(object, args):
+        object.get_status(args)
+        object.status_datadog(args)
+        return True
+
+    def remove_agents(object, args):
+
+        # if list of agents is not specified, restart all
+        if(not hasattr(args, 'list') or len(args.list) == 0):
+            args.list = object._docker_container_names
+
         client = docker.from_env()
         print("STOP: ")
         flag_success = True
-        for container_name in object._docker_container_names:
+        for container_name in args.list:
             print("   - " + container_name, end = ': ', flush=True)
             try:
                 container = client.containers.get(container_name)
@@ -178,7 +224,12 @@ class AgentLifeCycle:
                 flag_success = False
         return flag_success
 
-    def run(object, args):
+    def run_agents(object, args):
+
+        # if list of agents is not specified, restart all
+        if(not hasattr(args, 'list') or len(args.list) == 0):
+            args.list = object._docker_container_names
+
         object._agent_run_options = {}
         object._agent_run_options["cgs_agent"] = {"command": "start_cognicept_agent.py","volumes":{args.config.config_path + "agent/logs/": {"bind": "/root/.cognicept/agent/logs", "mode":"rw"}},"network_mode": "host"}
         object._agent_run_options["cgs_diagnostics_agent"] = {"command": "rosrun error_resolution_diagnoser error_resolution_diagnoser" ,"volumes":{args.config.config_path + "agent/logs/": {"bind": "/root/.cognicept/agent/logs", "mode":"rw"}} ,"network_mode": "host"}
@@ -197,9 +248,18 @@ class AgentLifeCycle:
         client = docker.from_env()
         print("RUN: ")
         success_flag = True
-        for container_name in object._docker_container_names:
+        for container_name in args.list:
             print("   - " + container_name, end = ': ', flush=True)
             try:
+                if(container_name not in object._docker_images):
+                    if("COG_AGENT_CONTAINERS" in args.config.config):
+                        containers = " (configured list: " + args.config.config["COG_AGENT_CONTAINERS"] +")"
+                    else:
+                        containers = ""
+                    print(bcolors.WARNING +  "NOT FOUND" + bcolors.ENDC + containers)
+                    success_flag = False
+                    continue
+
                 if(container_name in object._agent_run_options.keys()):
                     options = object._agent_run_options[container_name]
                 else:
@@ -222,6 +282,31 @@ class AgentLifeCycle:
                 success_flag = False
         return success_flag
 
+    def stop_datadog(object, args):
+        sts = subprocess.Popen(
+            "systemctl stop datadog-agent", shell=True).wait()
+        print("Health monitor agent stopped")
+        return True
+
+    def run_datadog(object, args):
+        sts = subprocess.Popen(
+            "systemctl start datadog-agent", shell=True).wait()
+        print("Health monitor agent started")
+        return True
+
+    def status_datadog(object, args):
+        sts = subprocess.Popen(
+            ["systemctl status datadog-agent", "-p"], shell=True, stdout=DEVNULL, stderr=DEVNULL).wait()
+        if sts == 0:
+            print("Health monitor status:" +
+                  bcolors.OKBLUE + " ACTIVE" + bcolors.ENDC)
+        elif sts ==4:
+            print("Health monitor status:" +
+                  bcolors.FAIL + " NOT FOUND" + bcolors.ENDC)
+        else:
+            print("Health monitor status:" +
+                  bcolors.WARNING + " INACTIVE" + bcolors.ENDC)
+
     def run_orbitty(object, args):
         os.system("xhost +local:root")
         client = docker.from_env()
@@ -251,44 +336,15 @@ class AgentLifeCycle:
 
     def update(object, args):
 
-        if(args.reset):
-            # reset credentials if requested
-            args.config.cognicept_login()
-        # retrieve jwt
-        token = args.config.get_cognicept_jwt()
+        # Attempt to login to ECR client
+        login_success = object.ecr_login(args)
 
-        headers = {"Authorization": "Bearer " + token}
-        try:
-            resp = requests.get( args.config.get_cognicept_api_uri() + "aws/assume_role/ecr", headers= headers, timeout = 5)
-
-            if resp.status_code != 200:
-                print('Cognicept REST API error: `{args.config.get_cognicept_api_uri()}` responded with {resp.status_code}')
-                return False
-        except requests.exceptions.Timeout:
-            print("Cognicept REST API error: time out.")
+        # If login failed, return false
+        if login_success is False:
+            print("You don't seem to have ECR repository permissions. Check your AWS credentials with the Cognicept team.")
             return False
-        except requests.exceptions.TooManyRedirects:
-            print("Cognicept REST API error: Wrong endpoint.")
-            return False
-        except:
-            print("Cognicept REST API error")
-            return False
-
-
-        ecr_client = boto3.client('ecr', region_name='ap-southeast-1', aws_access_key_id=resp.json()["AccessKeyId"], aws_secret_access_key = resp.json()["SecretAccessKey"], aws_session_token = resp.json()["SessionToken"])
+        
         print("Info: This may take a while depending on your connection.")
-
-        token = ecr_client.get_authorization_token()
-        username, password = base64.b64decode(token['authorizationData'][0]['authorizationToken']).decode().split(':')
-        registry = token['authorizationData'][0]['proxyEndpoint']
-
-        docker_client = docker.APIClient(base_url='unix://var/run/docker.sock')
-        try:
-            result = docker_client.login(username, password, registry=registry, reauth=True)
-        except docker.errors.APIError:
-            print("You don't have ECR repository permissions. Check you AWS credentials with Cognicept team.")
-            return False
-
         images = set(object._docker_images.values())
         N = len(images)
         i = 0
@@ -296,8 +352,7 @@ class AgentLifeCycle:
         for image_name in images:
             i = i + 1
             try:
-
-                for status in docker_client.pull(image_name, stream = True, decode = True):
+                for status in object.docker_client.pull(image_name, stream = True, decode = True):
                     if("progress" not in status):
                         status["progress"] = ""
                     print("[" + str(i) + "/" + str(N) + "]" + image_name + "-" + status["status"] + " " + status["progress"], end="")
@@ -308,3 +363,54 @@ class AgentLifeCycle:
 
         print("Info: Run `cognicept restart` to redeploy updated agents.")
         return success_flag
+
+    def construct_ecr_client(object, config_obj):
+        """
+        Utility function to create ECR client based on runtime.env credentials
+        """
+        # Get config
+        local_cfg = config_obj.config
+
+        if 'AWS_SESSION_TOKEN' in local_cfg:
+            object.ecr_client = boto3.client(
+                'ecr', region_name='ap-southeast-1',
+                aws_access_key_id=local_cfg['AWS_ACCESS_KEY_ID'],
+                aws_secret_access_key=local_cfg['AWS_SECRET_ACCESS_KEY'],
+                aws_session_token=local_cfg['AWS_SESSION_TOKEN'])
+        else:
+            object.ecr_client = boto3.client(
+                'ecr', region_name='ap-southeast-1',
+                aws_access_key_id=local_cfg['AWS_ACCESS_KEY_ID'],
+                aws_secret_access_key=local_cfg['AWS_SECRET_ACCESS_KEY'])
+
+    def ecr_login(object, args):
+        """
+        Utility function that is called to login to ECR
+        """
+        # Construct client
+        object.construct_ecr_client(args.config)
+        num_retries = 3
+
+        for trial in range(num_retries):
+            try:
+                # Get token
+                token = object.ecr_client.get_authorization_token()
+                # Parse username, password
+                username, password = base64.b64decode(
+                    token['authorizationData'][0]['authorizationToken']).decode().split(':')
+                # Get registry
+                registry = token['authorizationData'][0]['proxyEndpoint']
+                # Login
+                object.docker_client = docker.APIClient(
+                    base_url='unix://var/run/docker.sock')
+                object.docker_client.login(username, password,
+                                    registry=registry, reauth=True)
+                # Return true for successful login
+                return True
+            except (docker.errors.APIError, botocore.exceptions.ClientError):
+                # On failure, retry
+                print('Attempt #'+ str(trial+1) + bcolors.FAIL + " FAILED" + bcolors.ENDC + "\033[K")
+                # Wait for 1 second before retrying
+                time.sleep(1.0)
+        # If the loop is completed, login failed, so return false
+        return False
