@@ -26,6 +26,8 @@ def main():
     parser_status = subparsers.add_parser('status', help='Get status of Cognicept agents')
     parser_lastevent = subparsers.add_parser('lastevent', help='Display last event log reported by Cognicept agent')
     parser_update = subparsers.add_parser('update', help='Update Cognicept tools')
+    parser_keyrotate = subparsers.add_parser(
+        'keyrotate', help='Rotate Cognicept cloud keys')
     parser_restart = subparsers.add_parser('restart', help='Restart Cognicept agents')
     parser_start = subparsers.add_parser('start', help='Start Cognicept agents')
     parser_stop = subparsers.add_parser('stop', help='Stops Cognicept agents')
@@ -41,8 +43,12 @@ def main():
     parser_config.add_argument('--ssh',  help='Configure ssh access during remote intervention', action='store_true')
     parser_config.add_argument('--read',  help='Prints Cognicept configuration', action='store_true')
 
+    parser_keyrotate.set_defaults(func=local_cfg.cognicept_key_rotate)
+    parser_keyrotate.add_argument(
+        '--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
+
     agent_lifetime = AgentLifeCycle()
-    parser_status.set_defaults(func=agent_lifetime.get_status)
+    parser_status.set_defaults(func=agent_lifetime.status)
     parser_status.add_argument('--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
 
     parser_lastevent.set_defaults(func=agent_lifetime.get_last_event)
@@ -50,12 +56,21 @@ def main():
 
     parser_restart.set_defaults(func=agent_lifetime.restart)
     parser_restart.add_argument('--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
+    parser_restart.add_argument('list', help='List of agents to restart', metavar='list', type=str , nargs='*')
+    parser_restart.add_argument('--datadog', help='Restart only datadog agent', action='store_true')
+    parser_restart.add_argument('--agents', help='Restart only cognicept agents', action='store_true')
 
-    parser_start.set_defaults(func=agent_lifetime.run)
+    parser_start.set_defaults(func=agent_lifetime.start)
     parser_start.add_argument('--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
+    parser_start.add_argument('list', help='List of agents to start', metavar='list', type=str , nargs='*')
+    parser_start.add_argument('--datadog', help='Restart only datadog agent', action='store_true')
+    parser_start.add_argument('--agents', help='Restart only cognicept agents', action='store_true')
 
-    parser_stop.set_defaults(func=agent_lifetime.remove_agents)
+    parser_stop.set_defaults(func=agent_lifetime.stop)
     parser_stop.add_argument('--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
+    parser_stop.add_argument('list', help='List of agents to stop', metavar='list', type=str , nargs='*')
+    parser_stop.add_argument('--datadog', help='Stop only datadog agent', action='store_true')
+    parser_stop.add_argument('--agents', help='Stop only cognicept agents', action='store_true')
 
     parser_update.set_defaults(func=agent_lifetime.update)
     parser_update.add_argument('--path', help='Cognicept configuration directory (default: `' + DEFAULT_PATH + '`)', default=DEFAULT_PATH)
@@ -93,4 +108,4 @@ def main():
 
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
